# -*- coding:utf-8 -*-
# @Time     :2023/7/10 15:04
# @Author   :CHNJX
# @File     :adf.py
# @Desc     :
import logging
import time
from datetime import datetime

import pytest

from api_driver.template import Template


class TestResultPlugin:
    def __init__(self):
        self.results = {}
        self.test_count = 0
        self.pass_count = 0
        self.fail_count = 0
        self.skip_count = 0
        self.error_count = 0

    def pytest_collection_modifyitems(self, config, items):
        # 统计被跳过的测试用例数量
        for item in items:
            if item.get_closest_marker("skip"):
                self.skip_count += 1

    def pytest_runtest_logreport(self, report):
        if report.when == "call":
            self.test_count += 1

            # 统计测试结果数量
            if report.outcome == "passed":
                self.pass_count += 1
            elif report.outcome == "failed":
                self.fail_count += 1
            elif report.outcome == "error":
                self.error_count += 1

            testcase_info = report.nodeid.split('::')
            test_suite = testcase_info[0].split('/')[-1]
            test_class = testcase_info[1] if len(testcase_info) == 3 else None
            test_name = testcase_info[-1]
            result = {
                'test_model': test_suite,
                'test_class': test_class,
                'test_name': test_name,
                'result': report.outcome,
                'details': str(report.longrepr),
            }

            # 获取测试用例方法的日志信息
            logs = report.caplog if hasattr(report, 'caplog') else None
            # 将测试用例方法的日志添加到results中
            result['logs'] = logs

            if test_suite not in self.results:
                self.results[test_suite] = {}
            if test_class not in self.results[test_suite]:
                self.results[test_suite][test_class] = []
            self.results[test_suite][test_class].append(result)

    def get_results(self) -> dict:
        return {
            'test_count': self.test_count,
            'pass_count': self.pass_count,
            'fail_count': self.fail_count,
            'skip_count': self.skip_count,
            'error_count': self.error_count,
            'results': self.results
        }


class ApiDriver:

    def __init__(self):
        self.setup_logging()

    def setup_logging(self):
        logging.basicConfig(level=logging.INFO)

    def run_tests(self, testcases: str, report_file: str = None, thread_num: int = 0):
        """
        运行测试用例
        :param testcases: 测试用例路径
        :param report_file: 报告存在路径
        :param thread_num: 使用线程数
        :return:
        """
        plugin = TestResultPlugin()
        start_time = time.time()
        if thread_num:
            pytest.main([testcases, '-v', '-s', 'n', f'{thread_num}'], plugins=[plugin])
        else:
            pytest.main([testcases, '-v', '-s'], plugins=[plugin])
        end_time = time.time()
        execution_time = "{:.2f}s".format(end_time - start_time)
        res = plugin.get_results()
        res['execution_time'] = execution_time

        # 生成HTML报告
        if report_file:
            self.generate_html_report(res, report_file)

        return res

    def generate_html_report(self, results, report_file):
        report_content = self.get_html_content(results)
        with open(report_file, 'w', encoding='utf-8') as report_file:
            report_file.write(report_content)

    def get_html_content(self, results) -> str:
        # 获取当前日期和时间
        current_datetime = datetime.now()
        # 格式化输出当前日期和时间
        formatted_datetime = current_datetime.strftime('%Y-%m-%d %H:%M:%S')
        template = Template()
        return template.get_content('report_template.html',
                                    current=formatted_datetime,
                                    execution_time=results['execution_time'],
                                    results=results)



# -*- coding:utf-8 -*-
# @Time     :2023/8/20 10:38
# @Author   :CHNJX
# @File     :adf_proxy.py
# @Desc     :
from mitmproxy import options
from mitmproxy.proxy import ProxyConfig
from mitmproxy.tools.dump import DumpMaster


class InterceptAddon:
    def request(self, flow):
        print("Intercepted request:", flow.request.url)


if __name__ == "__main__":
    opts = options.Options(listen_host="0.0.0.0", listen_port=8080)  # 设置代理监听的地址和端口
    config = ProxyConfig(opts)
    master = DumpMaster(opts, with_termlog=False)

    addons = [
        InterceptAddon()
    ]

    master.server = ProxyServer(config)
    master.addons.add(*addons)

    try:
        print("Starting mitmproxy")
        master.run()
    except KeyboardInterrupt:
        master.shutdown()
        print("Mitmproxy has been shut down")



import json
import os
from datetime import datetime

from mitmproxy import ctx

from api_driver.command import har2case
from api_driver.har_parser import HarParser


class ExportFilter:
    def __init__(self):
        """
        :param host: 域名
        :param path: 路径
        """
        self.entries = []
        self.exclude_url = ['js', 'png', 'gif', 'jpg', 'css', 'jpeg', 'ico']

    def load(self, loader):
        loader.add_option(
            name="host",
            typespec=str,
            default='',
            help="Add a host as a url filter condition",
        )
        loader.add_option(
            name="path",
            typespec=str,
            default='',
            help="Add a path as a url filter condition",
        )
        loader.add_option(
            name="name",
            typespec=str,
            default='',
            help="Testcase name",
        )
        loader.add_option(
            name="testcase_path",
            typespec=str,
            default='',
            help="Storage path of testcase",
        )
        loader.add_option(
            name="exclude",
            typespec=str,
            default='',
            help="Path to be eliminated",
        )
        loader.add_option(
            name="api",
            typespec=str,
            default='',
            help="The path of the api-object",
        )
        loader.add_option(
            name="method",
            typespec=str,
            default='',
            help="The path of the api-object",
        )

    def response(self, flow):
        request_data = flow.request
        response_data = flow.response

        # 在这里使用传递的筛选关键词进行匹配
        host = ctx.options.host if ctx.options.host else ''
        path = ctx.options.path.split('|') if ctx.options.path else []
        method = ctx.options.method if ctx.options.method else ''
        if host not in request_data.pretty_url or (path and not any(p in request_data.pretty_url for p in path)):
            return

        if any(e in request_data.pretty_url for e in self.exclude_url):
            return
        # 对请求方法的过滤
        if method and request_data.method.upper() != method.upper():
            return

        started_datetime = datetime.fromtimestamp(request_data.timestamp_start)
        started_datetime_str = started_datetime.isoformat()

        entry = {
            "startedDateTime": started_datetime_str,
            "request": {
                "method": request_data.method,
                "url": request_data.url,
                "httpVersion": "HTTP/1.1",
                "cookies": [],
                "headers": [
                    {"name": h[0], "value": h[1]} for h in request_data.headers.items()
                ],
                "queryString": [],
                "postData": {},
                "headersSize": -1,
                "bodySize": len(request_data.content)
            },
            "response": {
                "status": response_data.status_code,
                "statusText": response_data.reason,
                "httpVersion": "HTTP/1.1",
                "cookies": [],
                "headers": [
                    {"name": h[0], "value": h[1]} for h in response_data.headers.items()
                ],
                "content": {
                    "size": len(response_data.content),
                    "mimeType": response_data.headers.get("Content-Type", ""),
                    "text": response_data.text  # 存储响应文本数据
                },
                "redirectURL": "",
                "headersSize": -1,
                "bodySize": len(response_data.content)
            },
            "cache": {},
            "timings": {},
            "serverIPAddress": "",
            "connection": "",
            "comment": ""
        }
        self.entries.append(entry)

    def done(self):
        har_data = {
            "log": {
                "version": "1.2",
                "creator": {
                    "name": "mitmproxy",
                    "version": "1.0"
                },
                "entries": self.entries
            }
        }
        testcase_name = ctx.options.name
        try:
            with open(f'har/{testcase_name}.har', 'w') as har_file:
                har_file.write(json.dumps(har_data, indent=2))
                har_file.flush()  # 确保数据被写入磁盘
        except Exception as e:
            # 写入失败，处理错误
            print(f"写入 HAR 文件时发生错误：{str(e)}")
            # 这里可以添加处理错误的逻辑
        else:
            # 写入成功，可以继续执行后续的代码
            # 调用 shell生成测试用例 命令
            testcase_path = ctx.options.testcase_path if ctx.options.testcase_path else 'testcase'
            api = ctx.options.api if ctx.options.api else ''
            exclude = ctx.options.exclude if ctx.options.exclude else ''
            har_file_path = os.path.join(os.getcwd(), f'har/{testcase_name}.har')
            har2case(['-h', har_file_path, '-t', testcase_path, '-a', api, '-e', exclude])
            # 这里可以添加后续代码逻辑


addons = [ExportFilter()]
